#!/usr/bin/env python
#
# This file is part of jfreespace.
#  
# Copyright (c) 2009-2012 Hillcrest Laboratories, Inc. 
# 
# jfreespace is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import sys
import getopt
import os

######################################################################
class MessageCodeGenerator:
    def writeMessages(self, messages):
        
        ### Create the package directories ###
        inReportPath = "./src/com/hcrest/jfreespace/inreport"
        if not os.path.isdir(inReportPath):
            os.mkdir(inReportPath)
        outReportPath = "./src/com/hcrest/jfreespace/outreport"
        if not os.path.isdir(outReportPath):
            os.mkdir(outReportPath)

        ### Create the decoder files ###
        ### The decoders determine what the message is based on ID and SubID
        decoderSet = [{'decoderName':'DecoderV0', 'subDecoderName':'SubDecoderV0', 'verName':'DecoderFactory.HID_MESSAGE_VERSION.BOOTLOADER', 'idIdx':0, 'subIdIdx':0},
                      {'decoderName':'DecoderV1', 'subDecoderName':'SubDecoderV1', 'verName':'DecoderFactory.HID_MESSAGE_VERSION.FSP16x', 'idIdx':0, 'subIdIdx':1},
                      {'decoderName':'DecoderV2', 'subDecoderName':'SubDecoderV2', 'verName':'DecoderFactory.HID_MESSAGE_VERSION.FSP2xx', 'idIdx':0, 'subIdIdx':4}]
        for v in range(0, 3):
            decoderFile = open(inReportPath + "/" + decoderSet[v]['decoderName'] + ".java", "w")
            self.writeFileHeader(decoderFile, 'inreport')
            self.writeDecoderFileHeader(decoderFile, decoderSet[v]['decoderName'], decoderSet[v]['idIdx'])
            self.writeDecoderCases(decoderFile, messages, decoderSet[v]['verName'], v)
            self.writeDecoderFileTrailer(decoderFile, decoderSet[v]['decoderName'])
            decoderFile.close()
        
            subDecoderFile = open(inReportPath + "/" + decoderSet[v]['subDecoderName'] + ".java", "w")
            self.writeFileHeader(subDecoderFile, 'inreport')
            self.writeDecoderFileHeader(subDecoderFile, decoderSet[v]['subDecoderName'], decoderSet[v]['subIdIdx'])
            self.writeSubDecoderCases(subDecoderFile, messages, decoderSet[v]['verName'], v)
            self.writeDecoderFileTrailer(subDecoderFile, decoderSet[v]['subDecoderName'])
            subDecoderFile.close()
        
        # Generate HidInMsgType.java and HidOutMsgType.java
        # Create an enum class for IN messages and one for OUT messages
        fileInfos = [(inReportPath, "HidInMsgType", False, "inreport"), (outReportPath, "HidOutMsgType", True, "outreport")]
        for fileInfo in fileInfos:
            hidTypeFile = open(fileInfo[0] + "/"+ fileInfo[1] + ".java", "w")
            self.writeFileHeader(hidTypeFile, fileInfo[3])
            self.writeMsgTypeFileStart(hidTypeFile, fileInfo[1])
            for message in messages:
                self.writeMsgTypeFileEntries(hidTypeFile, message, fileInfo[2])
            self.writeMsgTypeFileEnd(hidTypeFile)
            hidTypeFile.close()

        # Generate the HidInMsg classes
        for message in messages:
            if message.decode:
                self.writeInMessageClass(inReportPath, message)

        # Generate the HidOutMsg Classes
        for message in messages:
            if message.encode:
                self.writeOutMessageClass(outReportPath, message)

### General header code ##############################################
    def writeFileHeader(self, outHeader, package):
        # Print the struct, and define the message functions
        writeCopyright(outHeader)
        outHeader.write("\n")
        outHeader.write("package com.hcrest.jfreespace.")
        outHeader.write(package + ';')
        outHeader.write('\n')
        outHeader.write('\n')

### Decoder code ####################################################
    def writeDecoderFileHeader(self, outHeader, name, index):
        outHeader.write("public class ")
        outHeader.write(name)
        outHeader.write(" implements Decoder {\n")
        outHeader.write("""
    private static %s instance_ = null;
    private Decoder subDecoder_ = null;
    
    public HidInMsg decode(byte [] msg) {
        switch(msg[%d]) {
"""%(name, index))

    def writeDecoderCases(self, outFile, messages, versionName, v):
        usedIDs = []
        for message in messages:
            if not message.ID[v].has_key('constID'):
                continue # Skip messages that are not defined for this version
            if (not message.decode) or (message.ID[v]['constID'] in usedIDs):
                continue
            usedIDs.append(message.ID[v]['constID'])
            outFile.write("            case %d:\n"%message.ID[v]['constID'])
            if message.ID[v].has_key('subId'):
                outFile.write("                return subDecoder_.decode(msg);\n")
            else:
                outFile.write("                return new %s(msg, %s);\n"% (message.className, versionName))
                
    def writeSubDecoderCases(self, outFile, messages, versionName, v):
        for message in messages:
            if not (message.decode and message.ID[v].has_key('subId')):
                continue
            outFile.write("            case %d:\n"%message.ID[v]['subId']['id'])
            outFile.write("                return new %s(msg, %s);\n"% (message.className, versionName))
        
    def writeDecoderFileTrailer(self, outFile, name):
        outFile.write(
"""            default:
               return null;
        }
    }
    
    public static %s getInstance() {
        if (instance_ == null) {
            instance_ = new %s();
        }
        return instance_;
    }
    
    public void setSubDecoder(Decoder subDecoder) {
        subDecoder_ = subDecoder;
    }
}

"""% (name, name))
        
### Message type enum code ###########################################
    def writeMsgTypeFileStart(self, outFile, name):
        outFile.write("public enum ")
        outFile.write(name)
        outFile.write(" {\n")
    
    def writeMsgTypeFileEntries(self, outFile, message, encode):
        # encode decides which direction to test
        # False -> IN messages
        # True -> OUT messages
        if encode:
            if not message.encode:
                return
        else:
            if not message.decode:
                return
        outFile.write("   ")
        outFile.write(message.enumName)
        outFile.write(",\n")
        
    def writeMsgTypeFileEnd(self, outFile):
        outFile.write("}\n\n")
        
### HID IN Message classes code ######################################
    def writeInMessageClass(self, inPackage, message):
        fields = self.extractFields(message)
        cName = message.className
        classFile = open(inPackage + "/"+ cName + ".java", "w")
        self.writeFileHeader(classFile, 'inreport')
        self.writeInMsgClassHeader(classFile, cName)
        self.writeMsgClassFields(classFile, fields)
        self.writeMsgClassConstructor(classFile, message)
        self.writeMsgClassGetters(classFile, fields)
        for i in range(3):
            if len(message.Fields[i]) != 0:
                self.writeMsgClassDecoder(cName, classFile, fields, message.Fields[i], message.ID[i], i)
            else:
                self.writeMsgClassDummyDecoder(classFile, i)
        self.writeMsgClassTrailer(classFile)
        classFile.close()
        
    def extractFields(self, message):
        fields = {}
        for version in message.Fields:
            for field in version:
                if field['name'] == 'RESERVED':
                    continue
                if field.has_key('cType'):
                    if not fields.has_key(field['name']):
                        fields[field['name']] = self.cTypeToJavaType(field['cType'], field['size'])
                        if fields[field['name']]['warning'] == 'yes':
                            print ("Type problem found in message=>%s, field=>%s"%(message.name, field['name']))
                elif field.has_key('bits'):
                    for bit in field['bits']:
                        if bit['name'] == 'RESERVED':
                            continue
                        if not fields.has_key(bit['name']):
                            fields[bit['name']] = self.bitToJavaType(bit)
                elif field.has_key('nibbles'):
                    for nibble in field['nibbles']:
                        if nibble['name'] == 'RESERVED':
                            continue
                        if not fields.has_key(nibble['name']):
                            fields[nibble['name']] = {'type':'int', 'signed':False, 'length':4, 'count':1, 'warning':'no'}
        return fields
        
    def cTypeToJavaType(self, ct, sizeInBytes):
        typeInfo = {'type':'int', 'warning':'no'}
        if ct == 'uint32_t':
            typeInfo['signed'] = False
            typeInfo['count'] = sizeInBytes / 4
            typeInfo['width'] = 4
        elif ct == 'uint16_t':
            typeInfo['signed'] = False
            typeInfo['count'] = sizeInBytes / 2
            typeInfo['width'] = 2
        elif ct == 'uint8_t':
            typeInfo['signed'] = False
            typeInfo['count'] = sizeInBytes / 1
            typeInfo['width'] = 1
        elif ct == 'int32_t':
            typeInfo['signed'] = True
            typeInfo['count'] = sizeInBytes / 4
            typeInfo['width'] = 4
        elif ct == 'int16_t':
            typeInfo['signed'] = True
            typeInfo['count'] = sizeInBytes / 2
            typeInfo['width'] = 2
        elif ct == 'int8_t':
            typeInfo['signed'] = True
            typeInfo['count'] = sizeInBytes / 1
            typeInfo['width'] = 1
        else:
            print("ERROR: Unrecognized cType: %s"%ct)
            sys.exit(1)
        if sizeInBytes != typeInfo['count'] * typeInfo['width']:
            print("WARNING: Size(%d) is not a multiple of the size of cType(%s)"%(sizeInBytes, ct))
            typeInfo['warning'] = 'yes'
        return typeInfo
        
    def bitToJavaType(self, bt):
        if not bt.has_key('size'):
            return {'type':'boolean', 'signed':False, 'length':1, 'count':1, 'warning':'no'}
        elif bt['size'] == 1:
            return {'type':'boolean', 'signed':False, 'length':1, 'count':1, 'warning':'no'}
        else:
            return {'type':'int', 'signed':False, 'length':bt['size'], 'count':1, 'warning':'no'}
        
    def writeInMsgClassHeader(self, classFile, cName):
        classFile.write(
"""public class %s implements HidInMsg {
    
    private long timestamp_ = 0;
    private HidInMsgType msgType_;
    
    private int length_;
    private int dest_;
    private int src_;
    private String detailedString_ = "Not yet decoded.\\n";
"""%cName)
        
    def writeMsgClassFields(self, classFile, fields):
        for n, t in fields.iteritems():
            if t['count'] > 1:
                ary = "[] "
            else:
                ary = ""
            classFile.write("    private %s %s%s_;\n"%(t['type'], ary, n))
        classFile.write("\n")
        
    def writeMsgClassConstructor(self, classFile, message):
        classFile.write(
"""    public %s (byte [] msg, DecoderFactory.HID_MESSAGE_VERSION version) {

        msgType_ = HidInMsgType.%s;
        
        switch (version) {
            case BOOTLOADER:
                decodeV0(msg);
                break;
            case FSP16x:
                decodeV1(msg);
                break;
            case FSP2xx:
                decodeV2(msg);
                break;
        }
    }
    
    public HidInMsgType getMsgType() {
        return msgType_;
    }

    public String toDetailedString() {
        return detailedString_;
    }
"""% (message.className, message.enumName))
        
    def writeMsgClassGetters(self, classFile, fields):
        # Create getters for the always present header info
        classFile.write(
"""    public int getLength() {
        return length_;
    }
    
""")
        classFile.write(
"""    public int getDest() {
        return dest_;
    }
    
""")
        classFile.write(
"""    public int getSrc() {
        return src_;
    }
    
""")
        classFile.write(
"""    public long getTimestamp() {
        return timestamp_;
    }
    
""")
        classFile.write(
"""    public void  setTimestamp(long ts) {
        timestamp_ = ts;
    }
    
""")
        # Create getters based on fields present in message
        for n, t in fields.iteritems():
            method = n[0].capitalize() + n[1:]
            if t['count'] > 1:
                ary = "[] "
            else:
                ary = ""
            classFile.write(
"""    public %s %sget%s() {
        return %s_;
    }
    
"""%(t['type'], ary, method, n))
    
    def writeMsgClassDecoder(self, cName, classFile, allFields, someFields, ID, ver):
        # Method header
        classFile.write("""
    private void decodeV%d(byte [] msg) {
        StringBuffer b = new StringBuffer("%s:");
"""%(ver, cName))
        byteIndex = 1
        # HCOMM assignments if needed
        if ver == 2:
            byteIndex = 4
            classFile.write(
"""        length_ = msg[1];
        dest_ = msg[2];
        src_ = msg[3];
""")
        ## Account for sub ID if present
        if ID.has_key('subId'):
            byteIndex += 1
            
        # Decode each field
        for field in someFields:
            if field.has_key('synthesized'):
                    continue
            if field['name'] == 'RESERVED':
                byteIndex += field['size']
                continue
            if field.has_key('cType'):
                typeInfo = allFields[field['name']]
                if typeInfo['count'] > 1:
                    classFile.write("        %s_ = new int[%d];\n"%(field['name'], typeInfo['count']))
                    sizeOf = field['size'] / typeInfo['count']
                    for j in range(typeInfo['count']):
                        if typeInfo['signed']:
                            classFile.write("        %s_[%d] = DecoderFactory.byteArrayToInt(msg, %d, %d);\n"%(field['name'], j, byteIndex, sizeOf))
                            classFile.write("        b.append(\" %s[%d]: \" + %s_[%d]);\n"%(field['name'], j, field['name'], j))
                        else:
                            classFile.write("        %s_[%d] = DecoderFactory.byteArrayToUnsignedInt(msg, %d, %d);\n"%(field['name'], j, byteIndex, sizeOf))
                            classFile.write("        b.append(\" %s[%d]: \" + %s_[%d]);\n"%(field['name'], j, field['name'], j))
                        byteIndex += sizeOf 
                else:
                    if typeInfo['signed']:
                        classFile.write("        %s_ = DecoderFactory.byteArrayToInt(msg, %d, %d);\n"%(field['name'], byteIndex, field['size']))
                        classFile.write("        b.append(\" %s: \" + %s_);\n"%(field['name'], field['name']))
                    else:
                        classFile.write("        %s_ = DecoderFactory.byteArrayToUnsignedInt(msg, %d, %d);\n"%(field['name'], byteIndex, field['size']))
                        classFile.write("        b.append(\" %s: \" + %s_);\n"%(field['name'], field['name']))
                    byteIndex += field['size']
            elif field.has_key('bits'):
                bitCounter = 0
                for bit in field['bits']:
                    if bit['name'] != 'RESERVED':
                        if bit.has_key('size') and bit['size'] != 1:
                            classFile.write("        %s_ = (msg[%d] >> %d) & 0x%02X;\n"%(bit['name'], byteIndex, bitCounter, 2**bit['size']-1))
                            classFile.write("        b.append(\" %s: \" + %s_);\n"%(bit['name'], bit['name']))
                            bitCounter += bit['size']-1
                        else:
                            classFile.write("        %s_ = (msg[%d] & %d)  == %d ? true : false;\n"%(bit['name'], byteIndex, 2**bitCounter, 2**bitCounter))
                            classFile.write("        b.append(\" %s: \" + %s_);\n"%(bit['name'], bit['name']))
                            bitCounter += 1
                    else:
                        if bit.has_key('size'):
                            bitCounter += bit['size']
                        else:
                            bitCounter += 1
                byteIndex += field['size']
            elif field.has_key('nibbles'):
                nibbleCounter = 0
                for nibble in field['nibbles']:
                    if nibble['name'] != 'RESERVED':
                        classFile.write("        %s_ = (msg[%d] >> %d) & 0xf;\n"%(nibble['name'], byteIndex, nibbleCounter * 4))
                        classFile.write("        b.append(\" %s: \" + %s_);\n"%(nibble['name'], nibble['name']))
                    nibbleCounter += 1
                byteIndex += field['size']
        for field in someFields:
            if field.has_key('synthesized'):
                classFile.write(specialCaseCode(field['synthesized']))
        classFile.write("        b.append(\"\\n\");\n")
        classFile.write("        detailedString_ = b.toString();\n")
        classFile.write("    }\n")
    
    def writeMsgClassDummyDecoder(self, classFile, ver):
        classFile.write("""
    private void decodeV%d(byte [] msg) {
"""%ver)
        classFile.write(
"""        throw new RuntimeException("No decoder function for this message.");
""")
        classFile.write("    }\n")
        
    def writeMsgClassTrailer(self, classFile):
        classFile.write("}\n\n")
        
### HID OUT Message classes code #####################################
    def writeOutMessageClass(self, outPackage, message):
        fields = self.extractFields(message)
        cName = message.className
        eName = message.enumName
        classFile = open(outPackage + "/"+ cName + ".java", "w")
        self.writeFileHeader(classFile, 'outreport')
        self.writeOutMsgClassHeader(classFile, cName, message.ID)
        self.writeMsgClassFields(classFile, fields)
        self.writeOutMsgClassConstructor(classFile, cName, eName)
        self.writeMsgClassSetters(classFile, fields, message.ID)
        for i in range(3):
            if len(message.ID[i]) != 0:
                self.writeMsgClassEncoder(classFile, message.ID[i], fields, message.Fields[i], i)
            else:
                self.writeMsgClassDummyEncoder(classFile, message.ID[i], i)
        self.writeMsgClassTrailer(classFile)
        classFile.close()
    
    def writeOutMsgClassHeader(self, classFile, cName, msgID):
        classFile.write("public class %s extends HidOutMsg {\n\n"%(cName))
        if len(msgID[2]) > 0:
            classFile.write("    private int dest_ = 4;\n")
            
    def writeOutMsgClassConstructor(self, classFile, cName, eName):
        classFile.write("""    public %s() {
        super(HidOutMsgType.%s);
    }

"""%(cName, eName))
        
    def writeMsgClassSetters(self, classFile, fields, msgID):
        # Create setter for dest_ only if it is present
        if len(msgID[2]) > 0:
            classFile.write(
"""    @Override
    public void setDest(int dest) {
        dest_ = dest;
    }
    
""")
        else:
            classFile.write(
"""    @Override
    public void setDest(int dest) {
        // No action required
    }
    
""")
                
        # Create setters based on fields present in message
        for n, t in fields.iteritems():
            method = n[0].capitalize() + n[1:]
            if t['count'] > 1:
                classFile.write("    public void set%s(%s [] %s) {\n"%(method, t['type'], n))
                classFile.write("        %s_ = new %s[%d];\n"%(n, t['type'], t['count']))
                classFile.write("        System.arraycopy(%s, 0, %s_, 0, %s.length);\n"%(n, n, n))
                classFile.write("    }\n\n")
            else:
                classFile.write(
"""    public void set%s(%s %s) {
        %s_ = %s;
    }
    
"""%(method, t['type'], n, n, n))
        
    def writeMsgClassEncoder(self, classFile, ID, allFields, someFields, ver):
        classFile.write("\n    @Override\n    protected byte [] encodeV%d() {\n"%ver)
        msgLen = self.countBytesInFields(someFields)
        self.writeOutMsgHeader(classFile, ID, ver, msgLen)
        offset = {0:1, 1:1, 2:4} # dictionary mapping HID message protocol version to location where message specific field encoding begins
        offsetSubId = {0:2, 1:2, 2:5} # dictionary mapping HID message protocol version to location where message specific field encoding begins when subID is present
        if ID.has_key('subId'):
            self.encodeFields(classFile, allFields, someFields, offsetSubId[ver])
        else:
            self.encodeFields(classFile, allFields, someFields, offset[ver])
        classFile.write("    }\n")
        
    def writeMsgClassDummyEncoder(self, classFile, ID, ver):
        classFile.write("""
    @Override
    protected byte [] encodeV%d() {
        return null;
    }
"""%ver)
        
    def countBytesInFields(self, someFields):
        # Count the number of bytes used by message fields
        # Reserved fields in the middle are counted
        # Trailing reserved fields are not counted
        b = 0
        rsvd = 0
        for f in someFields:
            if f['name'] == 'RESERVED':
                rsvd += f['size'] # Keep track of reserved bytes for possible later use
            else:
                b += rsvd         # Since we encountered a valid field, add in any previous reserved bytes
                b += f['size']    # Add in the bytes for this field
                rsvd = 0          # Restart the reserved count
        return b
        
    def writeOutMsgHeader(self, classFile, ID, ver, msgLen):
        if ver == 0:
            v0ID2Len = {1:2, 2:70} # dictionary of report IDs to HID report length
            classFile.write("        byte [] msg = new byte[%d];\n"%v0ID2Len[ID['constID']])
            classFile.write("        msg[0] = %d;\n"%ID['constID']);
        elif ver == 1:
            v1ID2Len = {5:16, 7:8, 8:27, 9:2, 10:4, 15:31, 32:21, 33:23, 34:5} # dictionary of report IDs to HID report length
            classFile.write("        byte [] msg = new byte[%d];\n"%v1ID2Len[ID['constID']])
            classFile.write("        msg[0] = %d;\n"%ID['constID']);
            if ID.has_key('subId'):
                classFile.write("        msg[1] = %d;\n"%ID['subId']['id'])
        elif ver == 2:
            classFile.write("        byte [] msg = new byte[12];\n")
            classFile.write("        msg[0] = %d;\n"%ID['constID'])
            if ID.has_key('subId'):
                classFile.write("        msg[4] = %d;\n"%ID['subId']['id'])
            classFile.write("        msg[1] = %d;\n"%(msgLen + 5))
            classFile.write("        msg[2] = (byte) dest_;\n")
            classFile.write("        msg[3] = 0;\n")
        
    def encodeFields(self, classFile, allFields, someFields, offset):
        idx = offset
        for f in someFields:
            if f.has_key('synthesized'):
                    continue
            if f['name'] == 'RESERVED':
                idx += f['size']
                continue
            if f.has_key('cType'):
                typeInfo = allFields[f['name']]
                if typeInfo['count'] > 1:
                    sizeOf = f['size'] / typeInfo['count']
                    for c in range(typeInfo['count']):
                        for i in range(sizeOf):
                            classFile.write("        msg[%d] = (byte) (%s_[%d] >> %d);\n"%(idx, f['name'], c, 8 * i))
                            idx += 1
                else:
                    for i in range(f['size']):
                        classFile.write("        msg[%d] = (byte) (%s_ >> %d);\n"%(idx, f['name'], 8 * i))
                        idx += 1
            elif f.has_key('bits'):
                bitCnt = 0
                classFile.write("        msg[%d] = 0;\n"%(idx))
                for bit in f['bits']:
                    if bit['name'] != 'RESERVED':
                        classFile.write("        msg[%d] |= (%s_ ? 1 : 0) << %d;\n"%(idx, bit['name'], bitCnt))
                    bitCnt += 1
                idx += 1
            elif f.has_key('nibbles'):
                classFile.write("        msg[%d] = (byte) ((%s_  << 4) | %s_);\n"%(idx, f['nibbles'][1]['name'], f['nibbles'][0]['name']))
                idx += 1
        classFile.write("        return msg;\n")
        
# --------------------------  Syntax Helpers ------------------------------------

def fieldToPrintFormat(field):
    return '%d'

def IntConversionHelper(elementSize):
    if elementSize == 2:
        return "toInt"
    if elementSize == 4:
        return "toUInt32"
    raise "Unknown toIntXX size"
    
# ---------------------- Output printing helpers --------------------------------

def writeCopyright(outHeader):
    outHeader.write('''/*
 * This file is part of jfreespace.
 * 
 * Copyright (c) 2009-2012 Hillcrest Laboratories, Inc. 
 *
 * jfreespace is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
''')

#----------------------- Special Case Code ----------------------------
def specialCaseCode(case):
    if case == 'case_A':
        # Calculate the A value of the quaternion
        # A = sqrt(16384**2 - (B**2 + C**2 + D**2))
        specialCode = "        angularPosA_ = (int) Math.sqrt(268435456 - ((angularPosB_ * angularPosB_) + (angularPosC_ * angularPosC_) + (angularPosD_ * angularPosD_)));\n        b.append(\" angularPosA_: \" + angularPosA_);\n"
    else:
        print ("Unrecognized special case: %s" % case)
        specialCode =  "Unknown code goes here."
        
    return specialCode

# ---------------------- Main function --------------------------------
# Courtesy of Guido: http://www.artima.com/weblogs/viewpost.jsp?thread=4829
class Usage(Exception):
    def __init__(self, msg):
        self.msg = msg

def main(argv=None):
    if argv is None:
        argv = sys.argv
    try:
        try:			
            opts, args = getopt.getopt(argv[1:], ["h", "t"], ["help", "test"])
        except getopt.error, msg:
             raise Usage(msg)
        except AttributeError, msg:
             raise Usage(msg)

        # Use test mode to build BOTH encoders and decoders for ALL message types.
		# By default encoders are built for outgoing msgs and decoders for incoming msgs
        test = False
		
        # Process options
        for o, a in opts:
            if o in ("--h", "--help"):
                print __doc__
                return 0
            elif o in ("--t", "--test"):
                test = True		

        # process arguments
        messages = []
        g = {'test':test}
        d = {}

        for arg in args:
            execfile(arg, g, d)
            messages.extend(d['messages'])
        mcg = MessageCodeGenerator()
        mcg.writeMessages(messages)
    except Usage, err:
        print >>sys.stderr, err.msg
        print >>sys.stderr, "for help use --help"
        return 2
        
        
if __name__ == "__main__":
    __doc__ = "Options\n"
    __doc__ = __doc__ + "\t--h\t--help\tHelp menu\n"
    __doc__ = __doc__ + "\t--t\t--test\tGenerate encoders and decoders for all messages\n"
    __doc__ = __doc__ + "\n"
    __doc__ = __doc__ + "Arguments\n"
    __doc__ = __doc__ + "\t<file>\t*.py\tGenerate codecs & printers for messages in file"
    sys.exit(main())

