/*
 * This file is part of jfreespace.
 *
 * Copyright (c) 2009-2012 Hillcrest Laboratories, Inc.
 *
 * jfreespace is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */


package com.hcrest.jfreespace;

import com.hcrest.jfreespace.outreport.HidOutMsg;

/**
 * Interface to open, close, etc. and send messages to a Freespace device.
 * @author <a href="mailto:Joe.Tanen@Hillcrestlabs.com">Joe Tanen</a>
 * @version 1.0
 */
public interface DeviceAccessInterface {
    /**
     * Send a single message to the device and wait for transmission to complete.
     * handleSend will also be called on the registered listener when the message
     * has been sent.
     * @param msg The message to send.
     * @return FREESPACE_SUCCESS on success, error code on failure.
     */
    FreespaceErrorCodes sendMessage(HidOutMsg msg);
    
    /**
     * Send a single message to the device.  This method returns immediately.
     * handleSend will be called on the registered listener when the message
     * has been sent.
     * @param msg The message to send.
     * @return FREESPACE_SUCCESS on success, error code on failure.  Success is just getting the asynchronous message down to the lower layer, since this operation is not synchronous.
     */
     FreespaceErrorCodes sendMessageAsync(HidOutMsg msg);

    /**
     * Open this device.
     * @param listener The listener for the device.
     * @return FREESPACE_SUCCESS on success, error code on failure.
     */
    FreespaceErrorCodes open(DeviceListenerInterface listener);
    
    /**
     * Close this device.
     */
    void close();

    /**
     * Flush all send and receive messages currently enqueued.
     * @return FREESPACE_SUCCESS on success, false on failure.
     */
    FreespaceErrorCodes flush();

}
