/*
 * This file is part of jfreespace.
 *
 * Copyright (c) 2009-2012 Hillcrest Laboratories, Inc.
 *
 * jfreespace is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */


package com.hcrest.jfreespace;

import com.hcrest.jfreespace.inreport.HidInMsg;
import com.hcrest.jfreespace.outreport.HidOutMsg;

/**
 * Interface to receive updates on a specific Freespace device.
 * @author <a href="mailto:Matt.Liberty@Hillcrestlabs.com">Matt Liberty</a>
 * @version 1.0
 */
public interface DeviceListenerInterface {

    /**
     * Callback on a received message.
     * @param device The device that received the message. 
     * @param msg The received message.
     * @param rc The return code for the receive message.
     *   0 if successful, libfreespace error code if not.
     * @param ts A relative timestamp indicating when the message was received.
     */
    void handleRead(Device device, HidInMsg msg, int rc, long ts);

    /**
     * Callback on send message transmitted.
     * @param device The device to which the message was sent.
     * @param msg The sent message.
     * @param rc The return code for the sent message.
     *   0 if sent successfully, libfreespace error code on error.
     */
    void handleSend(Device device, HidOutMsg msg, int rc);
    
    /**
     * Callback when the device is removed.
     * @param device The device that was removed.
     */
    void notifyRemoved(Device device);
}
