/*
 * This file is part of jfreespace.
 *
 * Copyright (c) 2009-2012 Hillcrest Laboratories, Inc.
 *
 * jfreespace is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

package com.hcrest.jfreespace;

/**
 * The possible Freespace error codes returned by libfreespace.
 * @author <a href="mailto:Matt.Liberty@Hillcrestlabs.com">Matt Liberty</a>
 * @version 1.0
 */
public enum FreespaceErrorCodes {
    /** Success (no error) */
    FREESPACE_SUCCESS(0),
    
    /** Input/output error */
    FREESPACE_ERROR_IO(-1),
    
    /** Access denied (insufficient permissions) */
    FREESPACE_ERROR_ACCESS(-3),
    
    /** No such device (it may have been disconnected) */
    FREESPACE_ERROR_NO_DEVICE(-4),
    
    /** Entity not found */
    FREESPACE_ERROR_NOT_FOUND(-5),
    
    /** Resource busy */
    FREESPACE_ERROR_BUSY(-6),
    
    /** Operation timed out */
    FREESPACE_ERROR_TIMEOUT(-7),
    
    /** Pipe error */
    FREESPACE_ERROR_PIPE(-9),
    
    /** System call interrupted (perhaps due to signal) */
    FREESPACE_ERROR_INTERRUPTED(-10),
    
    /** Out of memory */
    FREESPACE_ERROR_OUT_OF_MEMORY(-11),

    /** Amount to send was larger than the max */
    FREESPACE_ERROR_SEND_TOO_LARGE(-20),

    /** Invalid or uninitialized device handle */
    FREESPACE_ERROR_INVALID_DEVICE(-21),

    /** Receive buffer was too small */
    FREESPACE_ERROR_RECEIVE_BUFFER_TOO_SMALL(-22),

    /** Unknown error when trying to create or start a thread */
    FREESPACE_ERROR_COULD_NOT_CREATE_THREAD(-23),

    /** Buffer was too small */
    FREESPACE_ERROR_BUFFER_TOO_SMALL(-24),

    /** No data was received */
    FREESPACE_ERROR_NO_DATA(-25),
    
    /** Could not encode the message type for that HID version. */
    FREESPACE_ERROR_COULD_NOT_ENCODE(-26),

    /** Any uncategorized or unplanned error */
    FREESPACE_ERROR_UNEXPECTED(-98);
    
    private final int ec_;
    private FreespaceErrorCodes(int ec) {
        ec_ = ec;
    }
    
    /**
     * Get the libfreespace integer error code for this error.
     * @return The interger error code.
     */
    public int getErrorCode() {
        return ec_;
    }
    
    /**
     * Get the instance with the specified integer error code.
     * @param ec The integer error code.
     * @return The instance corresponding to the integer error code.
     */
    public static FreespaceErrorCodes getInstance(int ec) {
        for (FreespaceErrorCodes that : FreespaceErrorCodes.values()) {
            if (that.ec_ == ec) {
                return that;
            }
        }
        return null;
    }

}
