/*
 * This file is part of jfreespace.
 *
 * Copyright (c) 2009-2012 Hillcrest Laboratories, Inc.
 *
 * jfreespace is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

package com.hcrest.jfreespace.outreport;

import com.hcrest.jfreespace.inreport.DecoderFactory;

public abstract class HidOutMsg {

    private HidOutMsgType msgType_;
    
    protected HidOutMsg(HidOutMsgType msgType) {
        msgType_ = msgType;
    }
    
    public byte [] getMsg(DecoderFactory.HID_MESSAGE_VERSION hVer){
        switch(hVer) {
            case BOOTLOADER:
                return encodeV0();
            case FSP16x:
                return encodeV1();
            case FSP2xx:
                return encodeV2();
            default:
                return null;
        }
    }
    
    public HidOutMsgType getMsgType() {
        return msgType_;
    }
    
    abstract public void setDest(int dest);
    
    abstract protected byte [] encodeV0();
    
    abstract protected byte [] encodeV1();
    
    abstract protected byte [] encodeV2();
}
